@@ -605,9 +605,11 @@ Finish:
           BB->isfloor = FALSE;
         else if(fabs(BB->upbo[K]-BB->UPbound) < intmargin)
           BB->isfloor = TRUE;
-        else
+        else {
+          BB->isfloor = TRUE;
           report(BB->lp, IMPORTANT, "fillbranches_BB: Inconsistent equal-valued bounds for %s\n",
                                     get_col_name(BB->lp, k));
+        }
       }
       if((BB->nodesleft == 1) &&
          ((BB->isfloor && (BB->UPbound >= lp->infinite)) ||
@@ -920,12 +922,18 @@ STATIC int solve_LP(lprec *lp, BBrec *BB)
 
     else if((lp->bb_totalnodes == 0) && (MIP_count(lp) > 0)) {
       if(lp->lag_status != RUNNING) {
-       report(lp, NORMAL, "\nRelaxed solution  " RESULTVALUEMASK " after %10.0f iter is B&B base.\n",
-                          lp->solution[0], (double) lp->total_iter);
+        report(lp, NORMAL, "\nRelaxed solution  " RESULTVALUEMASK " after %10.0f iter is B&B base.\n",
+                           lp->solution[0], (double) lp->total_iter);
         report(lp, NORMAL, " \n");
       }
-      if((lp->usermessage != NULL) && (lp->msgmask & MSG_LPOPTIMAL))
+      if((lp->usermessage != NULL) && (lp->msgmask & MSG_LPOPTIMAL)) {
+        REAL *best_solution = lp->best_solution;
+
+        /* transfer_solution(lp, TRUE); */
+        lp->best_solution = lp->solution;
         lp->usermessage(lp, lp->msghandle, MSG_LPOPTIMAL);
+        lp->best_solution = best_solution;
+      }
       set_var_priority(lp);
     }
 
@@ -1035,7 +1043,7 @@ STATIC int rcfbound_BB(BBrec *BB, int varno, MYBOOL isINT, REAL *newbound, MYBOO
 
 STATIC MYBOOL findnode_BB(BBrec *BB, int *varno, int *vartype, int *varcus)
 {
-  int    countsossc, countnint, k;
+  int    countsossc, countnint, k, reasonmsg = MSG_NONE;
   REAL   varsol;
   MYBOOL is_better = FALSE, is_equal = FALSE, is_feasible = TRUE;
   lprec  *lp = BB->lp;
@@ -1130,7 +1138,7 @@ STATIC MYBOOL findnode_BB(BBrec *BB, int *varno, int *vartype, int *varcus)
     /* Check if we have reached the depth limit for any individual variable
       (protects against infinite recursions of mainly integer variables) */
     k = *varno-lp->rows;
-    if((*varno > 0) && (lp->bb_varactive[k] >= abs(DEF_BB_LIMITLEVEL))) {
+    if((*varno > 0) && (lp->bb_limitlevel != 0) && (lp->bb_varactive[k] >= abs(lp->bb_limitlevel) /* abs(DEF_BB_LIMITLEVEL) */)) {
       /* if(!is_action(lp->nomessage, NOMSG_BBLIMIT)) {*/
 /*
         report(lp, IMPORTANT, "findnode_BB: Reached B&B depth limit %d for variable %d; will not dive further.\n\n",
@@ -1166,8 +1174,7 @@ STATIC MYBOOL findnode_BB(BBrec *BB, int *varno, int *vartype, int *varcus)
         if((lp->solutionlimit <= 0) || (lp->solutioncount < lp->solutionlimit)) {
           lp->solutioncount++;
           SETMIN(lp->bb_solutionlevel, lp->bb_level);
-          if((lp->usermessage != NULL) && (lp->msgmask & MSG_MILPEQUAL))
-            lp->usermessage(lp, lp->msghandle, MSG_MILPEQUAL);
+          reasonmsg = MSG_MILPEQUAL;
         }
       }
 
@@ -1184,18 +1191,18 @@ STATIC MYBOOL findnode_BB(BBrec *BB, int *varno, int *vartype, int *varcus)
         }
 
         if(lp->bb_trace ||
-           ((lp->verbose >= NORMAL) && (lp->print_sol == FALSE) && (lp->lag_status != RUNNING))) {
+           ((lp->verbose >= NORMAL) && ((lp->print_sol & 1) == FALSE) && (lp->lag_status != RUNNING))) {
           report(lp, IMPORTANT,
                  "%s solution " RESULTVALUEMASK " after %10.0f iter, %9.0f nodes (gap %.1f%%)\n",
                  (lp->bb_improvements == 0) ? "Feasible" : "Improved",
                  lp->solution[0], (double) lp->total_iter, (double) lp->bb_totalnodes,
                  100.0*fabs(my_reldiff(lp->solution[0], lp->bb_limitOF)));
         }
-        if((lp->usermessage != NULL) && (MIP_count(lp) > 0)) {
-          if((lp->msgmask & MSG_MILPFEASIBLE) && (lp->bb_improvements == 0))
-            lp->usermessage(lp, lp->msghandle, MSG_MILPFEASIBLE);
-          else if((lp->msgmask & MSG_MILPBETTER) && (lp->msgmask & MSG_MILPBETTER))
-            lp->usermessage(lp, lp->msghandle, MSG_MILPBETTER);
+        if(MIP_count(lp) > 0) {
+          if(lp->bb_improvements == 0)
+            reasonmsg = MSG_MILPFEASIBLE;
+          else
+            reasonmsg = MSG_MILPBETTER;
         }
 
         lp->bb_status = FEASFOUND;
@@ -1237,7 +1244,10 @@ STATIC MYBOOL findnode_BB(BBrec *BB, int *varno, int *vartype, int *varcus)
          ) {
       }
     }
-    if(lp->print_sol != FALSE) {
+    if((reasonmsg != MSG_NONE) && (lp->msgmask & reasonmsg) && (lp->usermessage != NULL))
+      lp->usermessage(lp, lp->msghandle, reasonmsg);
+
+    if((lp->print_sol & 1) != FALSE) {
       print_objective(lp);
       print_solution(lp, 1);
     }
@@ -1372,7 +1382,7 @@ STATIC MYBOOL post_BB(lprec *lp)
 /* This is the non-recursive B&B driver routine - beautifully simple, yet so subtle! */
 STATIC int run_BB(lprec *lp)
 {
-  BBrec *currentBB;
+  BBrec *currentBB, *currentBB0;
   int   varno, vartype, varcus, prevsolutions;
   int   status = NOTRUN;
 
@@ -1404,12 +1414,30 @@ STATIC int run_BB(lprec *lp)
     }
 #endif
 
+#define NewBB
+
+    currentBB0 = currentBB;
     if((status == OPTIMAL) && findnode_BB(currentBB, &varno, &vartype, &varcus))
+    {
       currentBB = push_BB(lp, currentBB, varno, vartype, varcus);
+#if defined NewBB
+      if (currentBB == currentBB0)
+      {
+        if(status == OPTIMAL)
+          status = INFEASIBLE;
+        if(lp->spx_status == OPTIMAL)
+          lp->spx_status = INFEASIBLE;
+      }
+#endif
+    }
 
-    else while((lp->bb_level > 0) && !nextbranch_BB(currentBB))
-      currentBB = pop_BB(currentBB);
-
+#if defined NewBB
+    if (currentBB == currentBB0)
+#else
+    else
+#endif
+      while((lp->bb_level > 0) && !nextbranch_BB(currentBB))
+        currentBB = pop_BB(currentBB);
   }
 
   /* Finalize */
